# Measurements
STAT_HEIGHT = 14
STAT_SPACING = 4
SEPARATOR_HEIGHT = 2
SEPARATOR_SPACING = 4
LINE_SPACING = 4
IMAGE_PADDING = 12

# Colors
UNIQUE_COLOR = (175, 96, 37)
ELE_COLOR = {'fire': (150, 0, 0),
             'cold': (54, 100, 146),
             'lightning': (255, 215, 0)}
CHAOS_COLOR = (208, 32, 144)
DESC_COLOR = (127, 127, 127)
PROP_COLOR = (136, 136, 255)
GEM_COLOR = (26, 162, 155)
NORMAL_COLOR = (170, 158, 130)
RARE_COLOR = (255, 255, 119)
PROPH_COLOR = (181, 75, 255)
CORRUPTED = (210, 0, 0)
CRAFTED = (192, 242, 255)
flavor_color = {"unique": UNIQUE_COLOR,
                "gem": GEM_COLOR,
                "normal": NORMAL_COLOR,
                "prophecy": PROPH_COLOR,
                "rare": RARE_COLOR,
                "magic": PROP_COLOR}